-------------------------------------------------------------------------------
--
-- Copyright (C) 2011
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: Multi page memory
-- Description:
--   When next_page_* pulses then the next access will occur in the next page.
-- Remarks:
-- . See common_paged_ram_crw_crw for details.

library IEEE, technology_lib;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;
library common_lib;
use work.common_pkg.all;
use technology_lib.technology_select_pkg.all;

entity common_paged_ram_rw_rw is
  generic (
    g_technology      : natural := c_tech_select_default;
    g_str             : string := "use_adr";
    g_data_w          : natural;
    g_nof_pages       : natural := 2;  -- >= 2
    g_page_sz         : natural;
    g_start_page_a    : natural := 0;
    g_start_page_b    : natural := 0;
    g_rd_latency      : natural := 1;
    g_true_dual_port  : boolean := true
  );
  port (
    rst         : in  std_logic;
    clk         : in  std_logic;
    clken       : in  std_logic := '1';
    next_page_a : in  std_logic;
    adr_a       : in  std_logic_vector(ceil_log2(g_page_sz) - 1 downto 0) := (others => '0');
    wr_en_a     : in  std_logic := '0';
    wr_dat_a    : in  std_logic_vector(g_data_w - 1 downto 0) := (others => '0');
    rd_en_a     : in  std_logic := '1';
    rd_dat_a    : out std_logic_vector(g_data_w - 1 downto 0);
    rd_val_a    : out std_logic;
    next_page_b : in  std_logic;
    adr_b       : in  std_logic_vector(ceil_log2(g_page_sz) - 1 downto 0) := (others => '0');
    wr_en_b     : in  std_logic := '0';
    wr_dat_b    : in  std_logic_vector(g_data_w - 1 downto 0) := (others => '0');
    rd_en_b     : in  std_logic := '1';
    rd_dat_b    : out std_logic_vector(g_data_w - 1 downto 0);
    rd_val_b    : out std_logic
  );
end common_paged_ram_rw_rw;

architecture str of common_paged_ram_rw_rw is
begin
  u_crw_crw : entity work.common_paged_ram_crw_crw
  generic map (
    g_technology     => g_technology,
    g_str            => g_str,
    g_data_w         => g_data_w,
    g_nof_pages      => g_nof_pages,
    g_page_sz        => g_page_sz,
    g_start_page_a   => g_start_page_a,
    g_start_page_b   => g_start_page_b,
    g_rd_latency     => g_rd_latency,
    g_true_dual_port => g_true_dual_port
  )
  port map (
    rst_a       => rst,
    rst_b       => rst,
    clk_a       => clk,
    clk_b       => clk,
    clken_a     => clken,
    clken_b     => clken,
    next_page_a => next_page_a,
    adr_a       => adr_a,
    wr_en_a     => wr_en_a,
    wr_dat_a    => wr_dat_a,
    rd_en_a     => rd_en_a,
    rd_dat_a    => rd_dat_a,
    rd_val_a    => rd_val_a,
    next_page_b => next_page_b,
    adr_b       => adr_b,
    wr_en_b     => wr_en_b,
    wr_dat_b    => wr_dat_b,
    rd_en_b     => rd_en_b,
    rd_dat_b    => rd_dat_b,
    rd_val_b    => rd_val_b
  );
end str;
