-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

library IEEE, technology_lib;
use IEEE.std_logic_1164.all;
use work.common_mem_pkg.all;
use technology_lib.technology_select_pkg.all;

entity common_ram_rw_rw is
  generic (
    g_technology : natural := c_tech_select_default;
    g_ram        : t_c_mem := c_mem_ram;
    g_init_file  : string := "UNUSED";
    g_true_dual_port : boolean := true
  );
  port (
    rst       : in  std_logic := '0';
    clk       : in  std_logic;
    clken     : in  std_logic := '1';
    wr_en_a   : in  std_logic := '0';
    wr_en_b   : in  std_logic := '0';
    wr_dat_a  : in  std_logic_vector(g_ram.dat_w - 1 downto 0) := (others => '0');
    wr_dat_b  : in  std_logic_vector(g_ram.dat_w - 1 downto 0) := (others => '0');
    adr_a     : in  std_logic_vector(g_ram.adr_w - 1 downto 0) := (others => '0');
    adr_b     : in  std_logic_vector(g_ram.adr_w - 1 downto 0) := (others => '0');
    rd_en_a   : in  std_logic := '1';
    rd_en_b   : in  std_logic := '1';
    rd_dat_a  : out std_logic_vector(g_ram.dat_w - 1 downto 0);
    rd_dat_b  : out std_logic_vector(g_ram.dat_w - 1 downto 0);
    rd_val_a  : out std_logic;
    rd_val_b  : out std_logic
  );
end common_ram_rw_rw;

architecture str of common_ram_rw_rw is
begin
  -- Use only one clock domain

  u_crw_crw : entity work.common_ram_crw_crw
  generic map (
    g_technology => g_technology,
    g_ram        => g_ram,
    g_init_file  => g_init_file,
    g_true_dual_port => g_true_dual_port
  )
  port map (
    rst_a     => rst,
    rst_b     => rst,
    clk_a     => clk,
    clk_b     => clk,
    clken_a   => clken,
    clken_b   => clken,
    wr_en_a   => wr_en_a,
    wr_en_b   => wr_en_b,
    wr_dat_a  => wr_dat_a,
    wr_dat_b  => wr_dat_b,
    adr_a     => adr_a,
    adr_b     => adr_b,
    rd_en_a   => rd_en_a,
    rd_en_b   => rd_en_b,
    rd_dat_a  => rd_dat_a,
    rd_dat_b  => rd_dat_b,
    rd_val_a  => rd_val_a,
    rd_val_b  => rd_val_b
  );
end str;
