-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-- Purpose: IP components declarations for various devices that get wrapped by the tech components

library IEEE, technology_lib;
use IEEE.std_logic_1164.all;
use technology_lib.technology_pkg.all;

package tech_fifo_component_pkg is
  -----------------------------------------------------------------------------
  -- ip_stratixiv
  -----------------------------------------------------------------------------

  component ip_stratixiv_fifo_sc is
  generic (
    g_use_eab    : string := "ON";
    g_dat_w      : natural;
    g_nof_words  : natural
  );
  port (
    aclr  : in std_logic;
    clock : in std_logic;
    data  : in std_logic_vector(g_dat_w - 1 downto 0);
    rdreq : in std_logic;
    wrreq : in std_logic;
    empty : out std_logic;
    full  : out std_logic;
    q     : out std_logic_vector(g_dat_w - 1 downto 0);
    usedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_stratixiv_fifo_dc is
  generic (
    g_dat_w      : natural;
    g_nof_words  : natural
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_stratixiv_fifo_dc_mixed_widths is
  generic (
    g_nof_words  : natural;  -- FIFO size in nof wr_dat words
    g_wrdat_w    : natural;
    g_rddat_w    : natural
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_wrdat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_rddat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words * g_wrdat_w / g_rddat_w) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  -----------------------------------------------------------------------------
  -- ip_arria10
  -----------------------------------------------------------------------------

  component ip_arria10_fifo_sc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic;
    clock   : in std_logic;
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdreq   : in std_logic;
    wrreq   : in std_logic;
    empty   : out std_logic;
    full    : out std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    usedw   : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_fifo_dc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_fifo_dc_mixed_widths is
  generic (
    g_nof_words : natural := 1024;  -- FIFO size in nof wr_dat words
    g_wrdat_w   : natural := 20;
    g_rddat_w   : natural := 10
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_wrdat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_rddat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words * g_wrdat_w / g_rddat_w) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  -----------------------------------------------------------------------------
  -- ip_arria10_e3sge3
  -----------------------------------------------------------------------------

  component ip_arria10_e3sge3_fifo_sc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic;
    clock   : in std_logic;
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdreq   : in std_logic;
    wrreq   : in std_logic;
    empty   : out std_logic;
    full    : out std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    usedw   : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_e3sge3_fifo_dc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_e3sge3_fifo_dc_mixed_widths is
  generic (
    g_nof_words : natural := 1024;  -- FIFO size in nof wr_dat words
    g_wrdat_w   : natural := 20;
    g_rddat_w   : natural := 10
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_wrdat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_rddat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words * g_wrdat_w / g_rddat_w) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  -----------------------------------------------------------------------------
  -- ip_arria10_e1sg
  -----------------------------------------------------------------------------

  component ip_arria10_e1sg_fifo_sc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic;
    clock   : in std_logic;
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdreq   : in std_logic;
    wrreq   : in std_logic;
    empty   : out std_logic;
    full    : out std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    usedw   : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_e1sg_fifo_dc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_e1sg_fifo_dc_mixed_widths is
  generic (
    g_nof_words : natural := 1024;  -- FIFO size in nof wr_dat words
    g_wrdat_w   : natural := 20;
    g_rddat_w   : natural := 10
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_wrdat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_rddat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words * g_wrdat_w / g_rddat_w) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  -----------------------------------------------------------------------------
  -- ip_arria10_e2sg
  -----------------------------------------------------------------------------

  component ip_arria10_e2sg_fifo_sc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic;
    clock   : in std_logic;
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdreq   : in std_logic;
    wrreq   : in std_logic;
    empty   : out std_logic;
    full    : out std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    usedw   : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_e2sg_fifo_dc is
  generic (
    g_use_eab   : string := "ON";
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_arria10_e2sg_fifo_dc_mixed_widths is
  generic (
    g_nof_words : natural := 1024;  -- FIFO size in nof wr_dat words
    g_wrdat_w   : natural := 20;
    g_rddat_w   : natural := 10
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_wrdat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_rddat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words * g_wrdat_w / g_rddat_w) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  -----------------------------------------------------------------------------
  -- ip_ultrascale
  -----------------------------------------------------------------------------

  component ip_ultrascale_fifo_sc is
  generic (
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic;
    clock   : in std_logic;
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdreq   : in std_logic;
    wrreq   : in std_logic;
    empty   : out std_logic;
    full    : out std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    usedw   : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_ultrascale_fifo_dc is
  generic (
    g_dat_w     : natural := 20;
    g_nof_words : natural := 1024
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_dat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_dat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

  component ip_ultrascale_fifo_dc_mixed_widths is
  generic (
    g_nof_words : natural := 1024;  -- FIFO size in nof wr_dat words
    g_wrdat_w   : natural := 20;
    g_rddat_w   : natural := 10
  );
  port (
    aclr    : in std_logic  := '0';
    data    : in std_logic_vector(g_wrdat_w - 1 downto 0);
    rdclk   : in std_logic;
    rdreq   : in std_logic;
    wrclk   : in std_logic;
    wrreq   : in std_logic;
    q       : out std_logic_vector(g_rddat_w - 1 downto 0);
    rdempty : out std_logic;
    rdusedw : out std_logic_vector(tech_ceil_log2(g_nof_words * g_wrdat_w / g_rddat_w) - 1 downto 0);
    wrfull  : out std_logic;
    wrusedw : out std_logic_vector(tech_ceil_log2(g_nof_words) - 1 downto 0)
  );
  end component;

end tech_fifo_component_pkg;
