-------------------------------------------------------------------------------
--
-- Copyright (C) 2014
-- ASTRON (Netherlands Institute for Radio Astronomy) <http://www.astron.nl/>
-- P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- The inferred Altera code was obtained using template insert with Quartus 14.0a10.

-- Quartus II VHDL Template
-- True Dual-Port RAM with dual clock
--
-- Read-during-write on port A or B returns newly written data
--
-- Read-during-write on port A and B returns unknown data.

library ieee;
use ieee.std_logic_1164.all;

entity ip_arria10_e2sg_true_dual_port_ram_dual_clock is
  generic
  (
    DATA_WIDTH : natural := 8;
    ADDR_WIDTH : natural := 6
  );
  port
  (
    clk_a : in std_logic;
    clk_b : in std_logic;
    addr_a  : in natural range 0 to 2**ADDR_WIDTH - 1;
    addr_b  : in natural range 0 to 2**ADDR_WIDTH - 1;
    data_a  : in std_logic_vector((DATA_WIDTH - 1) downto 0);
    data_b  : in std_logic_vector((DATA_WIDTH - 1) downto 0);
    we_a  : in std_logic := '1';
    we_b  : in std_logic := '1';
    q_a   : out std_logic_vector((DATA_WIDTH - 1) downto 0);
    q_b   : out std_logic_vector((DATA_WIDTH - 1) downto 0)
  );
end ip_arria10_e2sg_true_dual_port_ram_dual_clock;

architecture rtl of ip_arria10_e2sg_true_dual_port_ram_dual_clock is
  -- Build a 2-D array type for the RAM
  subtype word_t is std_logic_vector((DATA_WIDTH - 1) downto 0);
  type memory_t is array(2**ADDR_WIDTH - 1 downto 0) of word_t;

  -- Declare the RAM
  shared variable ram : memory_t;
begin
  -- Port A
  process(clk_a)
  begin
  if(rising_edge(clk_a)) then
    if(we_a = '1') then
      ram(addr_a) := data_a;
    end if;
    q_a <= ram(addr_a);
  end if;
  end process;

  -- Port B
  process(clk_b)
  begin
  if(rising_edge(clk_b)) then
    if(we_b = '1') then
      ram(addr_b) := data_b;
    end if;
    q_b <= ram(addr_b);
  end if;
  end process;
end rtl;
